.wrap [class*="CodeMirror-lint-marker"],
.wp-core-ui [class*="CodeMirror-lint-message"],
.wrap .CodeMirror-lint-marker-multiple {
	background-image: none;
}

.wp-core-ui .CodeMirror-lint-marker-error,
.wp-core-ui .CodeMirror-lint-marker-warning {
	cursor: help;
}

.wrap .CodeMirror-lint-marker-multiple {
	position: absolute;
	top: 0;
}

.wrap [class*="CodeMirror-lint-marker"]:before {
	font: normal 18px/1 dashicons;
	position: relative;
	top: -2px;
}

.wp-core-ui [class*="CodeMirror-lint-message"]:before {
	font: normal 16px/1 dashicons;
	left: 16px;
	position: absolute;
}

.wp-core-ui .CodeMirror-lint-message-error,
.wp-core-ui .CodeMirror-lint-message-warning {
	box-shadow: 0 1px 1px 0 rgba( 0, 0, 0, 0.1 );
	margin: 5px 0 2px;
	padding: 3px 12px 3px 28px;
}

.wp-core-ui .CodeMirror-lint-message-warning {
	background-color: #fff8e5;
	border-left: 4px solid #ffb900;
}

.wrap .CodeMirror-lint-marker-warning:before,
.wp-core-ui .CodeMirror-lint-message-warning:before {
	content: "\f534";
	color: #f6a306;
}

.wp-core-ui .CodeMirror-lint-message-error {
	background-color: #fbeaea;
	border-left: 4px solid #dc3232;
}

.wrap .CodeMirror-lint-marker-error:before,
.wp-core-ui .CodeMirror-lint-message-error:before {
	content: "\f153";
	color: #dc3232;
}

.wp-core-ui .CodeMirror-lint-tooltip {
	background: none;
	border: none;
	border-radius: 0;
	direction: ltr;
}

.wrap .CodeMirror .CodeMirror-matchingbracket {
	background: rgba(255, 150, 0, .3);
	color: inherit;
}

.CodeMirror {
	text-align: left;
}
